/**
 * @file
 * VideoJS player behavior for the player component.
 */
(function (Drupal) {
  /**
   * Track all VideoJS players on the page.
   */
  let allPlayers = [];

  /**
   * Pauses all other VideoJS players when one starts playing.
   *
   * @param {Object} currentPlayer - The current VideoJS player that is playing.
   */
  function pauseOtherPlayers(currentPlayer) {
    allPlayers.forEach(function (player) {
      // Only pause other players, not the one that's currently playing
      if (player !== currentPlayer && !player.paused()) {
        player.pause();
      }
    });
  }

  /**
   * Initialize VideoJS players with one-at-a-time playback behavior.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.videojsMediablockPlayer = {
    attach(context, settings) {
      // Reset the players list when the behavior is first attached
      if (context === document) {
        allPlayers = [];
      }

      // Find all VideoJS players in the context
      const videoElements = context.querySelectorAll('.video-js');

      videoElements.forEach(function (element) {
        // Check if this is a VideoJS player and not already processed
        if (
          element.player &&
          !element.hasAttribute('data-videojs-mediablock-processed')
        ) {
          const player = element.player;

          // Add this player to our tracked players list
          allPlayers.push(player);

          // Add play event listener
          player.on('play', function () {
            pauseOtherPlayers(this);
          });

          // Mark as processed to avoid duplicate processing
          element.setAttribute('data-videojs-mediablock-processed', 'true');
        }
        // Support for players that might initialize after this behavior runs
        else if (!element.hasAttribute('data-videojs-mediablock-processed')) {
          // Mark for processing
          element.setAttribute('data-videojs-mediablock-waiting', 'true');

          // Use MutationObserver to detect when videojs enhances this element
          const observer = new MutationObserver(function (mutations) {
            if (
              element.player &&
              element.hasAttribute('data-videojs-mediablock-waiting')
            ) {
              const player = element.player;

              // Add to our tracked players list
              allPlayers.push(player);

              // Add play event listener
              player.on('play', function () {
                pauseOtherPlayers(this);
              });

              // Mark as processed
              element.removeAttribute('data-videojs-mediablock-waiting');
              element.setAttribute('data-videojs-mediablock-processed', 'true');

              // Disconnect observer once processed
              observer.disconnect();
            }
          });

          // Observe the element for changes
          observer.observe(element, {
            attributes: true,
            childList: true,
            subtree: true,
          });
        }
      });

      // Approach for handling VideoJS players that are initialized after page load
      if (context === document) {
        document.addEventListener('videojs-player-ready', function (e) {
          if (
            e.detail &&
            e.detail.player &&
            !e.detail.player
              .el()
              .hasAttribute('data-videojs-mediablock-processed')
          ) {
            const player = e.detail.player;

            // Add to tracked players list
            allPlayers.push(player);

            // Add play event listener
            player.on('play', function () {
              pauseOtherPlayers(this);
            });

            // Mark as processed
            player
              .el()
              .setAttribute('data-videojs-mediablock-processed', 'true');
          }
        });
      }
    },
  };

  /**
   * Helper to initialize VideoJS players with SDC components.
   */
  if (typeof window.videojs !== 'undefined') {
    document.addEventListener('DOMContentLoaded', function () {
      document
        .querySelectorAll('video.video-js:not([data-videojs-initialized])')
        .forEach(function (videoElement) {
          if (typeof window.videojs !== 'undefined') {
            try {
              // Initialize VideoJS on the element
              const player = window.videojs(videoElement, {}, function () {
                // Player is ready
                this.el().setAttribute('data-videojs-initialized', 'true');

                // Dispatch a custom event that our behavior can listen for
                const event = new CustomEvent('videojs-player-ready', {
                  detail: { player: this },
                });
                document.dispatchEvent(event);
              });
            } catch (e) {
              console.error('Error initializing VideoJS player:', e);
            }
          }
        });
    });
  }
})(Drupal);
